@@ -446,7 +446,7 @@ glmnet.path <- function(x, y, weights=NULL, lambda = NULL, nlambda = 100,
 #' in the model.}
 #' \item{call}{The call that produced this object.}
 #' \item{nobs}{Number of observations.}
-#' \item{warm_fit}{If \code{save.fit=TRUE}, output of FORTRAN routine, used for
+#' \item{warm_fit}{If \code{save.fit=TRUE}, output of C++ routine, used for
 #' warm starts. For internal use only.}
 #' \item{family}{Family used for the model.}
 #' \item{converged}{A logical variable: was the algorithm judged to have
@@ -669,7 +669,7 @@ glmnet.fit <- function(x, y, weights, lambda, alpha = 1.0,
                 eta <- get_eta(x, start, start_int)
                 mu <- linkinv(eta <- eta + offset)
                 obj_val <- obj_function(y, mu, weights, family, lambda, alpha, start, vp)
-                if (trace.it == 2) cat("Iteration", iter, " Halved step 3, Objective:", 
+                if (trace.it == 2) cat("Iteration", iter, " Halved step 3, Objective:",
                                        obj_val, fill = TRUE)
             }
             halved <- TRUE
@@ -742,7 +742,7 @@ glmnet.fit <- function(x, y, weights, lambda, alpha = 1.0,
 #' in this package call \code{elnet.fit} as a subroutine. If a warm start object
 #' is provided, some of the other arguments in the function may be overriden.
 #'
-#' \code{elnet.fit} is essentially a wrapper around a FORTRAN subroutine which
+#' \code{elnet.fit} is essentially a wrapper around a C++ subroutine which
 #' minimizes
 #'
 #' \deqn{1/2 \sum w_i (y_i - X_i^T \beta)^2 + \sum \lambda \gamma_j
@@ -821,7 +821,7 @@ glmnet.fit <- function(x, y, weights, lambda, alpha = 1.0,
 #' Included for compability with glmnet output.}
 #' \item{call}{The call that produced this object.}
 #' \item{nobs}{Number of observations.}
-#' \item{warm_fit}{If \code{save.fit=TRUE}, output of FORTRAN routine, used for
+#' \item{warm_fit}{If \code{save.fit=TRUE}, output of C++ routine, used for
 #' warm starts. For internal use only.}
 #'
 elnet.fit <- function(x, y, weights, lambda, alpha = 1.0, intercept = TRUE,
@@ -862,6 +862,7 @@ elnet.fit <- function(x, y, weights, lambda, alpha = 1.0, intercept = TRUE,
         nx <- warm$nx
         r <- warm$r
         rsqc <- warm$rsqc
+        xv <- warm$xv
         vp <- warm$vp
     } else {
         nobs <- as.integer(nrow(x))
@@ -919,6 +920,7 @@ elnet.fit <- function(x, y, weights, lambda, alpha = 1.0, intercept = TRUE,
         nlp <- integer(1)
         r <- weights * y
         rsqc <- double(1)
+        xv <- double(nvars)
 
         # check if coefs were provided as warmstart: if so, use them
         if (!is.null(warm)) {
@@ -945,25 +947,22 @@ elnet.fit <- function(x, y, weights, lambda, alpha = 1.0, intercept = TRUE,
     thr <- as.double(thresh)
     v <- as.double(weights)
 
-    # take out components of x and run FORTRAN subroutine
+    a.new <- a
+    a.new[0] <- a.new[0] # induce a copy
+  
+    # take out components of x and run C++ subroutine
     if (inherits(x, "sparseMatrix")) {
         xm <- as.double(attr(x, "xm"))
         xs <- as.double(attr(x, "xs"))
-        ix <- as.integer(x@p + 1)
-        jx <- as.integer(x@i + 1)
-        x <- as.double(x@x)
-        wls_fit <- .Fortran("spwls",
-                            alm0=alm0,almc=almc,alpha=alpha,m=m,no=nobs,ni=nvars,
-                            x=x,ix=ix,jx=jx,xm=xm,xs=xs,r=r,v=v,intr=intr,ju=ju,vp=vp,cl=cl,nx=nx,thr=thr,
-                            maxit=maxit,a=a,aint=aint,g=g,ia=ia,iy=iy,iz=iz,mm=mm,
-                            nino=nino,rsqc=rsqc,nlp=nlp,jerr=jerr)
+        wls_fit <- spwls_exp(alm0=alm0,almc=almc,alpha=alpha,m=m,no=nobs,ni=nvars,
+                             x=x,xm=xm,xs=xs,r=r,xv=xv,v=v,intr=intr,ju=ju,vp=vp,cl=cl,nx=nx,thr=thr,
+                             maxit=maxit,a=a.new,aint=aint,g=g,ia=ia,iy=iy,iz=iz,mm=mm,
+                             nino=nino,rsqc=rsqc,nlp=nlp,jerr=jerr)
     } else {
-        x <- as.double(x)
-        wls_fit <- .Fortran("wls",
-                            alm0=alm0,almc=almc,alpha=alpha,m=m,no=nobs,ni=nvars,
-                            x=x,r=r,v=v,intr=intr,ju=ju,vp=vp,cl=cl,nx=nx,thr=thr,
-                            maxit=maxit,a=a,aint=aint,g=g,ia=ia,iy=iy,iz=iz,mm=mm,
-                            nino=nino,rsqc=rsqc,nlp=nlp,jerr=jerr)
+        wls_fit <- wls_exp(alm0=alm0,almc=almc,alpha=alpha,m=m,no=nobs,ni=nvars,
+                           x=x,r=r,xv=xv,v=v,intr=intr,ju=ju,vp=vp,cl=cl,nx=nx,thr=thr,
+                           maxit=maxit,a=a.new,aint=aint,g=g,ia=ia,iy=iy,iz=iz,mm=mm,
+                           nino=nino,rsqc=rsqc,nlp=nlp,jerr=jerr)
     }
 
     # if error code > 0, fatal error occurred: stop immediately
@@ -973,9 +972,12 @@ elnet.fit <- function(x, y, weights, lambda, alpha = 1.0, intercept = TRUE,
         stop(errmsg$msg, call. = FALSE)
     } else if (wls_fit$jerr < 0)
         return(list(jerr = wls_fit$jerr))
-    warm_fit <- wls_fit[c("almc", "m", "no", "ni", "r", "ju", "vp", "cl", "nx",
+    warm_fit <- wls_fit[c("almc", "r", "xv", "ju", "vp", "cl", "nx",
                           "a", "aint", "g", "ia", "iy", "iz", "mm", "nino",
                           "rsqc", "nlp")]
+    warm_fit[['m']] <- m
+    warm_fit[['no']] <- nobs
+    warm_fit[['ni']] <- nvars
     class(warm_fit) <- "warmfit"
 
     beta <- Matrix::Matrix(wls_fit$a, sparse = TRUE)
@@ -1221,10 +1223,10 @@ get_eta <- function(x, beta, a0) {
 }
 
 #' Helper function to compute weighted mean and standard deviation
-#' 
-#' Helper function to compute weighted mean and standard deviation. 
+#'
+#' Helper function to compute weighted mean and standard deviation.
 #' Deals gracefully whether x is sparse matrix or not.
-#' 
+#'
 #' @param x Observation matrix.
 #' @param weights Optional weight vector.
 #'
@@ -1234,7 +1236,7 @@ get_eta <- function(x, beta, a0) {
 weighted_mean_sd <- function(x, weights=rep(1,nrow(x))){
     weights <- weights/sum(weights)
     xm <- drop(t(weights)%*%x)
-    xv <- drop(t(weights)%*%x^2)-xm^2
-    xv[abs(xv) < 10*.Machine$double.eps] <- 0
+    xv <- drop(t(weights)%*%scale(x,xm,FALSE)^2)
+    xv[xv < 10*.Machine$double.eps] <- 0
     list(mean = xm, sd = sqrt(xv))
 }
